@@ -1,12 +1,13 @@
 # ABSTRACT: Tidy YAML files
 use strict;
+use warnings;
 use warnings FATAL => qw/ substr /;
 
 use v5.20;
 use experimental qw/ signatures /;
 package YAML::Tidy;
 
-our $VERSION = '0.003_001'; # TRIAL VERSION
+our $VERSION = '0.005_001'; # TRIAL VERSION
 
 use YAML::Tidy::Node;
 use YAML::Tidy::Config;
@@ -111,11 +112,8 @@ sub _process($self, $parent, $node) {
             }
             else {
                 if ($parent->{type} eq 'MAP' and not $node->{index} % 2) {
-                    # zero indented sequence
-                    if ($indent == 1) {
-                        $indent = 2;
-                    }
-                    $indent -= 2;
+                    # zero indented sequence?
+                    $indent = $self->cfg->indent_seq_in_map;
                 }
             }
 
@@ -132,16 +130,16 @@ sub _process($self, $parent, $node) {
     }
     else {
         my $ignore_firstlevel = ($self->partial and $level == 0);
-        my $multiline = $node->multiline;
-        if ($parent->{type} eq 'MAP' and ($node->{index} % 2 and not $multiline)) {
-            return;
-        }
         if ($node->empty_scalar) {
             return;
         }
         if ($node->{name} eq 'alias_event') {
             return;
         }
+        if ($parent->{type} eq 'MAP' and ($node->{index} % 2 and not $node->multiline)) {
+            $self->_replace_quoting($node);
+            return;
+        }
         my $new_indent = $parent->indent + $indent;
         my $new_spaces = ' ' x $new_indent;
 
@@ -156,7 +154,12 @@ sub _process($self, $parent, $node) {
         my $before = substr($line, 0, $col);
         if ($before =~ tr/ \t//c) {
             # same line as key
-            $before =~ s/[\t ]+$/ /;
+            my $remove = 0;
+            $before =~ s/([\t ]+)$/ / and $remove = -1 + length $1;
+            $node->open->{column} -= $remove;
+            unless ($node->multiline) {
+                $node->close->{column} -= $remove;
+            }
             $line = $before . substr($line, $col);
             $lines->[ $startline ] = $line;
             $skipfirst = 1;
@@ -174,7 +177,14 @@ sub _process($self, $parent, $node) {
                 unless ($line =~ tr/ //c) {
                     next;
                 }
-                $line =~ s/^ */$new_spaces/;
+                my $remove = 0;
+                $line =~ s/^( *)/$new_spaces/ and $remove = length($1) - length($new_spaces);
+                if ($i == $startline) {
+                    $node->open->{column} -= $remove;
+                    unless ($node->multiline) {
+                        $node->close->{column} -= $remove;
+                    }
+                }
                 $lines->[ $i] = $line;
             }
         }
@@ -255,29 +265,110 @@ sub _process($self, $parent, $node) {
             if ($node->empty_scalar) {
                 return;
             }
-            $startline++ if $skipfirst;
-            $endline = $node->close->{line};
-            return if $startline >= @$lines;
-            my $line = $lines->[ $startline ];
-            my ($sp) = $line =~ m/^( *)/;
-            if ($ignore_firstlevel) {
-                $new_indent = length $sp;
-                $new_spaces = ' ' x $new_indent;
-            }
-            my @slice = @$lines[$startline .. $endline ];
-            if ($level == 0 and not $indenttoplevelscalar) {
-                $new_spaces = ' ' x ($new_indent - $indent);
-            }
-            for my $line (@slice) {
-                if ($line =~ tr/ //c) {
-                    $line =~ s/^[\t ]*/$new_spaces/;
+            my $remove = 0;
+            if (not $skipfirst or $node->multiline) {
+                my $startline = $startline;
+                $startline++ if $skipfirst;
+                $endline = $node->close->{line};
+                return if $startline >= @$lines;
+                my $line = $lines->[ $startline ];
+                my ($sp) = $line =~ m/^( *)/;
+                if ($ignore_firstlevel) {
+                    $new_indent = length $sp;
+                    $new_spaces = ' ' x $new_indent;
                 }
+                my @slice = @$lines[$startline .. $endline ];
+                if ($level == 0 and not $indenttoplevelscalar) {
+                    $new_spaces = ' ' x ($new_indent - $indent);
+                }
+                for my $line (@slice) {
+                    if ($line =~ tr/ //c) {
+                        $line =~ s/^([\t ]*)/$new_spaces/
+                            and $remove = length($1) - length($new_spaces);
+                    }
+                }
+                $node->close->{column} -= $remove;
+                @$lines[$startline .. $endline ] = @slice;
+            }
+            if (not $node->multiline) {
+                $self->_replace_quoting($node);
             }
-            @$lines[$startline .. $endline ] = @slice;
         }
     }
 }
 
+my $RE_INT_CORE = qr{^([+-]?(?:[0-9]+))$};
+my $RE_FLOAT_CORE = qr{^([+-]?(?:\.[0-9]+|[0-9]+(?:\.[0-9]*)?)(?:[eE][+-]?[0-9]+)?)$};
+my $RE_INT_OCTAL = qr{^0o([0-7]+)$};
+my $RE_INT_HEX = qr{^0x([0-9a-fA-F]+)$};
+my @null = (qw/ null NULL Null ~ /, '');
+my @true = qw/ true TRUE True /;
+my @false = qw/ false FALSE False /;
+my @inf = qw/ .inf .Inf .INF +.inf +.Inf +.INF  -.inf -.Inf -.INF /;
+my @nan = qw/ .nan .NaN .NAN /;
+my @re = ($RE_INT_CORE, $RE_INT_OCTAL, $RE_INT_HEX,  $RE_FLOAT_CORE);
+my $re = join '|', @re;
+my @all = (@null, @true, @false, @inf, @nan);
+
+sub _replace_quoting($self, $node) {
+    # TODO nodes with tags or anchors
+    my $default_style = $self->cfg->default_scalar_style;
+    # single line flow scalars
+    if (defined $default_style and $node->{style} != $default_style) {
+        my ($changed, $new_string) = $self->_change_style($node, $default_style);
+        if ($changed) {
+            my $lines = $self->{lines};
+            my $line = $lines->[ $node->open->{line} ];
+            my ($from, $to) = ($node->open->{column}, $node->close->{column});
+            if (defined $node->{anchor} or $node->{tag}) {
+                my ($anchor, $tag, $comments, $scalar) = $self->_find_scalar_start($node);
+                $from = $scalar->[1];
+            }
+            substr($line, $from, $to - $from, $new_string);
+            my $diff = length($new_string) - ($to - $from);
+            if ($diff) {
+                $self->{tree}->_move_columns($node->open->{line}, $node->close->{column} + 1, $diff);
+            }
+            $node->close->{column} += $diff;
+            $lines->[ $node->open->{line} ] = $line;
+        }
+    }
+}
+
+sub _change_style($self, $node, $style) {
+    my $value = $node->{value};
+    if (grep { $_ eq $value } @all or $value =~ m/($re)/) {
+        # leave me alone
+        return (0);
+    }
+    else {
+        my $emit = $self->_emit_value($value, $style);
+        chomp $emit;
+        return (0) if $emit =~ tr/\n//;
+        my $first = substr($emit, 0, 1);
+        my $new_style =
+            $first eq "'" ? YAML_SINGLE_QUOTED_SCALAR_STYLE
+            : $first eq '"' ? YAML_DOUBLE_QUOTED_SCALAR_STYLE
+            : YAML_PLAIN_SCALAR_STYLE;
+        if ($new_style eq $style) {
+            return (1, $emit);
+        }
+    }
+    return (0);
+}
+
+sub _emit_value($self, $value, $style) {
+    my $options = {};
+    my $events = [
+        { name => 'stream_start_event' },
+        { name => 'document_start_event', implicit => 1 },
+        { name => 'scalar_event', style => $style, value => $value },
+        { name => 'document_end_event', implicit => 1 },
+        { name => 'stream_end_event' },
+    ];
+    return YAML::LibYAML::API::XS::emit_string_events($events, $options);
+}
+
 sub _process_doc($self, $parent, $node) {
     DEBUG and say STDERR "_process_doc($node)";
     my $lines = $self->{lines};
@@ -429,6 +520,7 @@ sub _process_flow_scalar($self, $parent, $node, $block_indent) {
 }
 
 sub _find_scalar_start($self, $node) {
+#    warn __PACKAGE__.':'.__LINE__.": ========= _find_scalar_start $node\n";
     my $lines = $self->{lines};
     my $from = $node->line;
     my $to = $node->realendline;
@@ -453,7 +545,7 @@ sub _find_scalar_start($self, $node) {
             next;
         }
         my $cur;
-        while ($part =~ m/\G\s*([&!])(\S+)/g) {
+        while ($part =~ m/\G\s*([&!])(\S*)/g) {
             my $type = $1;
             my $name = $2;
             $cur = pos $part;
@@ -484,6 +576,7 @@ sub _find_scalar_start($self, $node) {
         }
     }
     $scalar ||= [$to, length($slice[ -1 ]), ''];
+#    warn __PACKAGE__.':'.__LINE__.$".Data::Dumper->Dump([\$scalar], ['scalar']);
     return ($anchor, $tag, \@comments, $scalar);
 }
 
